// polynomial.cpp source file

#include "polynomial.h"		// include polynomial class declaration

// constructor
polynomial::polynomial()	
{
	// every polynomial will start out as 0x^0 so we set this up accordingly
	m_head = new node;
	m_head->coeff = 0;
	m_head->exp = 0;
	m_head->next = NULL;
	m_head->prev = NULL;

	// at the start of the polynomial, 
	// the tail will be pointing to the same place as the head
	m_tail = m_head;
}
polynomial::~polynomial()
{
	// perform clean up on the polynomial
}

void polynomial::InsertTerm(int _coeff, unsigned int _exp)
{
	// there are basically 3 possible cases when inserting a term into a polynomial:
	// either the exponent being inserted is greater than the exponent of m_tail,
	// the exponent being inserted is less than the exponent of m_tail,
	// or the exponent being inserted is equal to m_tail

	// first case: exp being inserted is greater than polynomial's greatest exp so far
	if (_exp > m_tail->exp)
	{
		// if the exp being inserted is greater than the polynomial's current greatest exp,
		// we need to insert 0x^E for all the terms in between.  For example:
		// if the user inserts 2x^5, we also need to insert 0x^4, 0x^3, 0x^2, and 0x^1

		// determine the value of the first exponent being inserted
		int nExpCounter = m_tail->exp;

		// detemine the number of insertions to be made. For example:
		// if p1 = 1x^1 + 5, and 2x^5 is inserted, we need to make
		// (exp being inserted) - (p1's largest exp) insertions, or (5 - 1) = 4 insertions.
		// specifically: 0x^4, 0x^3, 0x^2, and the one the user requested, 2x^5
		int nNumInsertions = (_exp - m_tail->exp);

		// insert the first nNumInsertions - 1 insertions w/ the coefficient of 0
		for (int i = 0; i < nNumInsertions - 1; i++)
		{
			nExpCounter++;
			m_tail->next = new node;
			m_tail->next->coeff = 0;
			m_tail->next->exp = nExpCounter;
			m_tail->next->next = NULL;
			m_tail->next->prev = m_tail;
			m_tail = m_tail->next;
		}
		
		// insert the last insertion w/ the coeff and exp the user requested
		m_tail->next = new node;
		m_tail->next->coeff = _coeff;
		m_tail->next->exp = _exp;
		m_tail->next->next = NULL;
		m_tail->next->prev = m_tail;
		m_tail = m_tail->next;
	}

	// case 2: exponent being inserted is less than the polynomial's greatest exp so far
	else if (_exp < m_tail->exp)
	{
		// since the exponent being inserted is less than the polynomial's current
		// greatest exponent, the node into which we need to insert the term already exists.
		// all we need to do is traverse to the node whose exponent = the exponent whose 
		// coefficient is going to be changed.
		// once we're there, we simply replace the current coefficient w/ the new coefficient

		node* pVisit = m_head;		// pointer used to traverse the list

		while (pVisit != NULL)
		{
			if (_exp == pVisit->exp)
				pVisit->coeff = _coeff;
			pVisit = pVisit->next;
		}

		if (pVisit != NULL)
			delete pVisit;				// delete the visit pointer when we're done w/ it
	}

	// case 3: exponent being inserted = the polynomial's greatest exponent so far
	else
	{
		m_tail->coeff = _coeff;		// simply replace the current coeff w/ the new coeff
	}
}

polynomial polynomial::operator+ (polynomial& pol)
{
	// p1 and p2 are copies of each polynomials so we do not alter the contents of the originals
	// sum is where the sum of p1 and p2 will be stored
	polynomial sum, p1, p2;

	p1 = *this;		// p1 = current polynomial class (p1 in the expression p1 + p2)
	p2 = pol;		// p2 = polynomial passed as a parameter (p2 in the expression p1 + p2) 

	// determine which polynomial has the highest exponent and insert 0x^0 terms in front
	// of the smaller polynomial to ensure that both polynomials are the same size
	if (p1.GetTail()->exp > p2.GetTail()->exp)
	{
		p2.InsertTerm(0, p1.GetTail()->exp);
	}
	else if (p1.GetTail()->exp < p2.GetTail()->exp)
	{
		p1.InsertTerm(0, p2.GetTail()->exp);
	}

	// create two visit pointers to traverse each polynomial
	// add the cofficient of each polynomial and insert it as
	// as a new term into the polynomial, sum
	int nExpCounter = 0;
	node* pVisit1 = p1.GetHead();
	node* pVisit2 = p2.GetHead();
	while (pVisit1 != NULL)
	{
		sum.InsertTerm(pVisit1->coeff + pVisit2->coeff, nExpCounter);
		pVisit1 = pVisit1->next;
		pVisit2 = pVisit2->next;
		nExpCounter++;
	}

	// perform "clean up"
	if (pVisit1 != NULL)
		delete pVisit1;
	if (pVisit2 != NULL)
		delete pVisit2;

	// return the sum of p1 + p2
	return sum;
}

node* polynomial::GetHead() const
{
	// returns a pointer to the head of the polynomial
	return m_head;
}

node* polynomial::GetTail() const
{
	// returns a pointer to the tail of the polynomial
	return m_tail;
}

int polynomial::GetEval(int x)
{
	// traverse the polynomial and calculate its value for any given integer, x
	int result = 0;
	node* pVisit = m_head;
	while (pVisit != NULL)
	{
		// the static_cast<int> is used just to avoid a compiler warning
		// in reality i know all data is integers so now accuracy will be lost
		result += pVisit->coeff * static_cast<int>(pow(x, pVisit->exp));
		pVisit = pVisit->next;
	}

	// perform "clean up"
	if (pVisit != NULL)
		delete pVisit;

	return result;
}

string polynomial::GetPolyAsString()
{
	// We traverse the list backwards so that the polynomial will be represented
	// in order of decreasing exponents

	node* pVisit = m_tail;		// a visit pointer for traversing the list
	string returnString;		// the string to be returned
	char coeffBuffer[10];		// temp string which stores the coeff as a string
	char expBuffer[10];			// temp string which stores the exp as a string
	bool bFirstTime = true;		// determines if it's the first time through the do loop

	do
	{
		// if the coefficient is 0, we do not want to include 
		// that term in the string representation of the polynomial
		if (pVisit->coeff != 0)
		{
			// we need a '+' between each term
			if (!bFirstTime)
				returnString += " + ";

			// convert the coefficient to a string
			_itoa(pVisit->coeff, coeffBuffer, 10);
			returnString += coeffBuffer;

			// if the exponent is a one, we simply display an x, not x^1
			if (pVisit->exp == 1)
				returnString += "x";
			else if (pVisit->exp > 1)
			{
				_itoa(pVisit->exp, expBuffer, 10);
				returnString += "x^";
				returnString += expBuffer;
			}

			bFirstTime = false;			// no longer the first time through the loop
		}

		pVisit = pVisit->prev;			// move to previous node in the list
	}
	while (pVisit != NULL);

	if (pVisit != NULL)
		delete pVisit;

	return returnString;
}
